package adapter;

import java.util.NoSuchElementException;


/*
 * Copyright (c) 1997, 2006, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/**
 * An iterator over a collection.  {@code Iterator} takes the place of
 * {@link Enumeration} in the Java Collections Framework.  Iterators
 * differ from enumerations in two ways:
 *
 * <ul>
 *	 <li> Iterators allow the caller to remove elements from the
 *		  underlying collection during the iteration with well-defined
 *		  semantics.
 *	 <li> Method names have been improved.
 * </ul>
 *
 * <p>This interface is a member of the
 * <a href="{@docRoot}/../technotes/guides/collections/index.html">
 * Java Collections Framework</a>.
 *
 * @author  Josh Bloch
 * @see Collection
 * @see ListIterator
 * @see Iterable
 * @since 1.2
 */
public class IteratorAdapter
implements Iterator
{

	/**
	* Tiene traccia del vettore cui si riferisce. Attenzione: non dell'adapter,
	* ma del vettore sottostante.
	*/
	private Vector v = null;

	/**
	* Riferimento all'adapter cui questo iteratore si riferisce.
	* Utilizzato per controllare la consistenza delle mutazioni dell'adapter.
	*/
	private Adapter a = null;
	
	/**
	* Tiene traccia della posizione dell'elemento next.
	*/
	private int pos = 0;

	/**
	* Flag esecuzione next, permette la successiva remove.
	*/
	private boolean nextDone = false;

	/**
	* Ultimo modificatore dell'adapter al tempo della mia istanziazione
	*/
	private Object lv;

	/**
	* Contatore delle operazioni di modifica dell'adapter al momento
	* della mia istanziazione.
	*/
	private long cop;

	protected IteratorAdapter(Vector theV, Adapter theA)
	{
		v = theV;
		a = theA;
		pos = 0;
		lv = a.obj;
		cop = a.countOp;
	}

	private void checkValid()
	{
		if(cop != a.countOp && a.obj != this)
		{
			throw new IllegalStateException("Adapter modificato dopo la mia creazione");
		}
	}
	
	/**
	* Returns {@code true} if the iteration has more elements.
	* (In other words, returns {@code true} if {@link #next} would
	* return an element rather than throwing an exception.)
	*
	* @return {@code true} if the iteration has more elements
	*/
	public boolean hasNext()
	{
		checkValid();
		assert pos >= 0;

		return pos < v.size();
	}

	/**
	* Returns the next element in the iteration.
	*
	* @return the next element in the iteration
	* @throws NoSuchElementException if the iteration has no more elements
	*/
	public Object next()
	{
		checkValid();
		if(pos >= v.size())
		{
			throw new NoSuchElementException("List has less than " + (pos+1) + " elements");
		}

		nextDone = true;
		return v.elementAt(pos++);	
	}

	/**
	* Removes from the underlying collection the last element returned
	* by this iterator (optional operation).  This method can be called
	* only once per call to {@link #next}.  The behavior of an iterator
	* is unspecified if the underlying collection is modified while the
	* iteration is in progress in any way other than by calling this
	* method.
	*
	* @throws UnsupportedOperationException if the {@code remove}
	*		operation is not supported by this iterator
	*
	* @throws IllegalStateException if the {@code next} method has not
	*		yet been called, or the {@code remove} method has already
	*		been called after the last call to the {@code next}
	*		method
	*/
	public void remove()
	{
		checkValid();
		if(!nextDone)
		{
			throw new IllegalStateException(this.toString());
		}

		nextDone = false;
		v.removeElementAt(--pos);
		a.setMe(this);
	}
}
